import * as React from "react";
import { useState } from "react";
import {
  FolderRegular,
  DocumentRegular,
  DocumentPdfRegular,
  VideoRegular,
  ChevronDown16Filled,
  TextField16Regular,
  Calendar16Regular,
  SelectAllOn16Regular,
  CommentAdd16Regular,
  MoreHorizontal16Filled,
} from "@fluentui/react-icons";
import {
  DataGridBody,
  DataGridRow,
  DataGrid,
  DataGridHeader,
  DataGridHeaderCell,
  DataGridCell,
  TableCellLayout,
  TableColumnDefinition,
  createTableColumn,
  Button,
  Menu,
  MenuTrigger,
  MenuList,
  MenuItem,
  MenuPopover,
  Tooltip,
} from "@fluentui/react-components";

const items = [
  {
    姓名: { label: "陈丹", icon: <DocumentRegular /> },
    性别: { label: "女", status: "available" },
    年龄: { label: "18岁", timestamp: 1 },
    最后在线时间: { label: "2024/11/6 13:50" },
  },
  {
    姓名: { label: "蔡青松", icon: <FolderRegular /> },
    性别: { label: "男", status: "busy" },
    年龄: { label: "30岁", timestamp: 2 },
    最后在线时间: { label: "2024/11/6 13:50" },
  },
  {
    姓名: { label: "杨东明", icon: <VideoRegular /> },
    性别: { label: "男", status: "away" },
    年龄: { label: "25岁", timestamp: 2 },
    最后在线时间: { label: "2024/11/6 13:50" },
  },
  {
    姓名: { label: "吴瑞强", icon: <DocumentPdfRegular /> },
    性别: { label: "男", status: "offline" },
    年龄: { label: "25岁", timestamp: 3 },
    最后在线时间: { label: "2024/11/6 13:50" },
  },
];

const columns = [
  createTableColumn({
    columnId: "姓名",
    compare: (a, b) => {
      return a.姓名.label.localeCompare(b.姓名.label);
    },
    renderHeaderCell: () => {
      return (
        <TableCellLayout media={<TextField16Regular />}>
          姓名
          <Menu>
            <MenuTrigger disableButtonEnhancement>
              <Button
                appearance="subtle"
                icon={<ChevronDown16Filled />}
              ></Button>
            </MenuTrigger>

            <MenuPopover>
              <MenuList>
                <MenuItem>New </MenuItem>
                <MenuItem>New Window</MenuItem>
                <MenuItem disabled>Open File</MenuItem>
                <MenuItem>Open Folder</MenuItem>
              </MenuList>
            </MenuPopover>
          </Menu>
        </TableCellLayout>
      );
    },
    renderCell: (item) => {
      return ChangeBtnShowType(item);
    },
  }),
  createTableColumn({
    columnId: "性别",
    compare: (a, b) => {
      return a.性别.label.localeCompare(b.性别.label);
    },
    renderHeaderCell: () => {
      return (
        <TableCellLayout media={<SelectAllOn16Regular />}>
          性别
          <Menu>
            <MenuTrigger disableButtonEnhancement>
              <Button
                appearance="subtle"
                icon={<ChevronDown16Filled />}
              ></Button>
            </MenuTrigger>

            <MenuPopover>
              <MenuList>
                <MenuItem>New </MenuItem>
                <MenuItem>New Window</MenuItem>
                <MenuItem disabled>Open File</MenuItem>
                <MenuItem>Open Folder</MenuItem>
              </MenuList>
            </MenuPopover>
          </Menu>
        </TableCellLayout>
      );
    },
    renderCell: (item) => {
      return item.性别.label;
    },
  }),
  createTableColumn({
    columnId: "年龄",
    compare: (a, b) => {
      return a.年龄.timestamp - b.年龄.timestamp;
    },
    renderHeaderCell: () => {
      return (
        <TableCellLayout media={<TextField16Regular />}>
          年龄
          <Menu>
            <MenuTrigger disableButtonEnhancement>
              <Button
                appearance="subtle"
                icon={<ChevronDown16Filled />}
              ></Button>
            </MenuTrigger>

            <MenuPopover>
              <MenuList>
                <MenuItem>New </MenuItem>
                <MenuItem>New Window</MenuItem>
                <MenuItem disabled>Open File</MenuItem>
                <MenuItem>Open Folder</MenuItem>
              </MenuList>
            </MenuPopover>
          </Menu>
        </TableCellLayout>
      );
    },

    renderCell: (item) => {
      return item.年龄.label;
    },
  }),
  createTableColumn({
    columnId: "最后在线时间",
    compare: (a, b) => {
      return a.最后在线时间.label.localeCompare(b.最后在线时间.label);
    },
    renderHeaderCell: () => {
      return (
        <TableCellLayout media={<Calendar16Regular />}>
          最后在线时间
          <Menu>
            <MenuTrigger disableButtonEnhancement>
              <Button
                appearance="subtle"
                icon={<ChevronDown16Filled />}
              ></Button>
            </MenuTrigger>

            <MenuPopover>
              <MenuList>
                <MenuItem>New </MenuItem>
                <MenuItem>New Window</MenuItem>
                <MenuItem disabled>Open File</MenuItem>
                <MenuItem>Open Folder</MenuItem>
              </MenuList>
            </MenuPopover>
          </Menu>
        </TableCellLayout>
      );
    },
    renderCell: (item) => {
      return (
        <TableCellLayout media={item.最后在线时间.icon}>
          {item.最后在线时间.label}
        </TableCellLayout>
      );
    },
  }),
];

/**
 * 鼠标移动到姓名列上时，显示两个按钮
 * @param item 包含姓名标签的对象
 * @returns 返回 JSX 元素
 */
const ChangeBtnShowType = (item) => {
  const [showButton, setShowButton] = useState(false);
  const handleMouseEnter = () => {
    setShowButton(true);
  };

  const handleMouseLeave = () => {
    setShowButton(false);
  };

  return (
    <div
      style={{
        width: "100%",
        display: "flex",
        justifyContent: "space-between",
        alignItems: "center",
      }}
      onMouseEnter={handleMouseEnter}
      onMouseLeave={handleMouseLeave}
    >
      {item.姓名.label}
      {showButton && (
        <div>
          <Tooltip content="更多操作" positioning="below">
            <Button
              appearance="subtle"
              icon={<MoreHorizontal16Filled />}
              onclick={() => {
                alert("更多操作");
              }}
            ></Button>
          </Tooltip>
          <Tooltip content="添加注释" positioning="below">
            <Button appearance="subtle" icon={<CommentAdd16Regular />}></Button>
          </Tooltip>
        </div>
      )}
    </div>
  );
};

const List = () => {
  const defaultSelectedItems = React.useMemo(() => new Set([1]), []);

  return (
    <DataGrid
      items={items}
      columns={columns}
      selectionMode="single"
      // sortable
      subtleSelection
      resizableColumns
      defaultSelectedItems={defaultSelectedItems}
      style={{ minWidth: "550px" }}
    >
      <DataGridHeader>
        <DataGridRow>
          {({ renderHeaderCell }) => (
            <DataGridHeaderCell>{renderHeaderCell()}</DataGridHeaderCell>
          )}
        </DataGridRow>
      </DataGridHeader>
      <DataGridBody>
        {({ item, rowId }) => (
          <DataGridRow
            key={rowId}
            selectionCell={{ radioIndicator: { "aria-label": "Select row" } }}
          >
            {({ renderCell }) => (
              <DataGridCell>{renderCell(item)}</DataGridCell>
            )}
          </DataGridRow>
        )}
      </DataGridBody>
    </DataGrid>
  );
};

export default List;
